import { randomUUID } from 'node:crypto'
import { BaseEntity } from '@/application/base/baseEntity'
import type { UUID } from '@/application/types'
import { SpriteAction } from '@/entities/spriteAction'
import { Collection, Entity, OneToMany, PrimaryKey, Property } from '@mikro-orm/core'

export class BaseSprite extends BaseEntity {
  @PrimaryKey()
  id = randomUUID()

  @Property()
  name!: string

  @OneToMany({ mappedBy: 'sprite', orphanRemoval: true })
  spriteActions = new Collection<SpriteAction>(this)

  @Property({ nullable: true })
  width: number | null = null

  @Property({ nullable: true })
  height: number | null = null

  @Property()
  createdAt = new Date()

  @Property()
  updatedAt = new Date()

  setId(id: UUID) {
    this.id = id
    return this
  }

  getId() {
    return this.id
  }

  setName(name: string) {
    this.name = name
    return this
  }

  getName() {
    return this.name
  }

  setSpriteActions(spriteActions: Collection<SpriteAction>) {
    this.spriteActions = spriteActions
    return this
  }

  getSpriteActions() {
    return this.spriteActions
  }

  setWidth(width: number | null) {
    this.width = width
    return this
  }

  getWidth() {
    return this.width
  }

  setHeight(height: number | null) {
    this.height = height
    return this
  }

  getHeight() {
    return this.height
  }

  setCreatedAt(createdAt: Date) {
    this.createdAt = createdAt
    return this
  }

  getCreatedAt() {
    return this.createdAt
  }

  setUpdatedAt(updatedAt: Date) {
    this.updatedAt = updatedAt
    return this
  }

  getUpdatedAt() {
    return this.updatedAt
  }
}
