import { BaseEvent } from '@/application/base/baseEvent'
import { SocketEvent } from '@/application/enums'
import type { UUID } from '@/application/types'
import { Sprite } from '@/entities/sprite'
import { SpriteAction } from '@/entities/spriteAction'
import SpriteRepository from '@/repositories/spriteRepository'

interface CopyPayload {
  id: UUID
}

export default class SpriteCopyEvent extends BaseEvent {
  public listen(): void {
    this.socket.on(SocketEvent.GM_SPRITE_COPY, this.handleEvent.bind(this))
  }

  private async handleEvent(payload: CopyPayload, callback: (success: boolean) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const spriteRepository = new SpriteRepository()
      const sourceSprite = await spriteRepository.getById(payload.id)

      if (!sourceSprite) {
        this.logger.error('gm:sprite:copy error', 'Source sprite not found')
        return callback(false)
      }

      // Populate source sprite with spriteActions
      await spriteRepository.getEntityManager().populate(sourceSprite, ['spriteActions'])

      const newSprite = new Sprite()
      await newSprite.setName(`${sourceSprite.getName()} (Copy)`).save()

      for (const spriteAction of sourceSprite.getSpriteActions()) {
        const newSpriteAction = new SpriteAction()
        await newSpriteAction
          .setSprite(newSprite)
          .setAction(spriteAction.getAction())
          .setSprites(spriteAction.getSprites() ?? [])
          .setOriginX(spriteAction.getOriginX())
          .setOriginY(spriteAction.getOriginY())
          .setFrameWidth(spriteAction.getFrameWidth())
          .setFrameHeight(spriteAction.getFrameHeight())
          .setFrameRate(spriteAction.getFrameRate())
          .save()
      }

      return callback(true)
    } catch (error) {
      this.logger.error(`Error copying sprite:`, String(error))
      return callback(false)
    }
  }
}
